/*
           panteltje downcount2srt

         Copyright (C) Jan Panteltje 2009-always

    email: panteltje@yahoo.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#define VERSION "0.2"

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <ctype.h>
#include <string.h>

#define TEXT_ARRAY_SIZE 4096
#define MAX_LINES 40

int debug_flag;


void print_usage()
{
fprintf(stderr, "Usage:\n\
downcount2srt [-d] -h hour -m minute -s second [-t lead_text] [-u trail_text] [-v end_text] [-x display_hor] [-y display_minute] [-z display-second]> my_movie.srt.\n\
-d                  activates debug mode.\n\
-h hour             start hour count down counter.\n\
-m minute           start minute coun tdown counter.\n\
-s second           start second count down counter.\n\
-t lead_text        text displayed before or above the count down counter, a '\\n' creates a CR LF sequence.\n\
-u trail_text       text displayed after or below the count down counter, a '\\n' creates a CR LF sequence.\n\
-v end_text         text displayed when the count down counter reaches zero, a '\\n' creates a CR LF sequence.\n\
-x display_hour     hour counter start is movie, default 0\n\
-y display_minute   minute counter starts in movie, default 0\n\
-z display_second   second counter start in movie, default 0\n\
Use the created .srt file with mplayer for example like this (see man mplayer):\n\
mplayer my_movie.avi -sub my_movie.srt -subfont-text-scale 50 -font /usr/share/fonts/wine/arial.ttf\n\
Example, the '\\n' causes text to go to a new line:\n\
downcount2srt -h 0 -m 0 -s 12 -t \"Flight 51 to mars\\nTime to launch\\n\" -u \"\\nPlease put on your space suits\\nboarding gate 51\" -v \"\\nFlight 51 has left\\nYou missed your flight\" > my_movie.srt\n"\
);

} /* end function print_usage */



int main(int argc, char **argv)
{
int a, c, i, j;
int hour, minute, second;
int display_hour, display_minute, display_second, display_start_milli_second, display_end_milli_second;
int start_seconds, seconds_to_go;
int entry_number;
char lead_text[TEXT_ARRAY_SIZE];
char trail_text[TEXT_ARRAY_SIZE];
char end_text[TEXT_ARRAY_SIZE];
char temp1[MAX_LINES][TEXT_ARRAY_SIZE];
char temp2[MAX_LINES][TEXT_ARRAY_SIZE];
char temp3[MAX_LINES][TEXT_ARRAY_SIZE];
int lf_flag;
int h_flag, m_flag, s_flag;

/* defaults */
debug_flag = 0;
/* this is where we start in the movie, could be made a command line option */
display_hour = 0;
display_minute = 0;
display_second = 0;

display_start_milli_second = 0;
display_end_milli_second = 999;

/* end defaults */


h_flag = 0;
m_flag = 0;
s_flag = 0;
while(1)
	{
	a = getopt(argc, argv, "dh:m:s:t:u:v:x:y:z:");
	if(a == -1) break;
	switch(a)
		{
		case 'd': // debug
			debug_flag = 1;
			break;
		case 'h': // hour
			hour = atoi(optarg);
			h_flag = 1;
			break;
		case 'm': // minute
			minute = atoi(optarg);
			m_flag = 1;
			break;
		case 's': // second
			second = atoi(optarg);
			s_flag = 1;
			break;
		case 't': // extra text before counter
			sprintf(lead_text, "%s", optarg);
			break;
		case 'u': // extra text after counter
			sprintf(trail_text, "%s", optarg);
			break;
		case 'v': // start hour in movie
			sprintf(end_text, "%s", optarg);
			break;
		case 'x': // start hour in movie
			display_hour = atoi(optarg);
			break;
		case 'y': // start minute in movie
			display_minute = atoi(optarg);
			break;
		case 'z': // start second in movie
			display_second = atoi(optarg);
			break;
        case -1:
        	break;
		case '?':
			if (isprint(optopt) )
 				{
 				fprintf(stderr, "Unknown option `-%c'.\n", optopt);
 				}
			else
				{
				fprintf(stderr, "Unknown option character `\\x%x'.\n",\
				optopt);
				}
			print_usage();
			exit(1);
			break;			
		default:
			print_usage();
			exit(1);
			break;
		} /* end switch a */
	} /* end while getopt() */


/* Identify */
fprintf(stderr, "\nPanteltje(c) downcount2srt-%s\n", VERSION);

/* argument check */
if((!h_flag) || (!m_flag) || (!s_flag) )
	{
	print_usage();
	
	exit(1);
	}

/* sanity checks */
if( (hour < 0) || (hour > 23) )
	{
	fprintf(stderr, "hour should be 0-23, aborting.\n");

	print_usage();
	exit(1);
	}
if((minute < 0) || (minute > 59) )
	{
	fprintf(stderr, "minute should be 0-59, aborting\n");
	
	print_usage();
	exit(1);
	}

if((second < 0) || (second > 59) )
	{
	fprintf(stderr, "second should be 0-59, aborting.\n");
	
	print_usage();
	exit(1);
	}


if( (display_hour < 0) || (display_hour > 23) )
	{
	fprintf(stderr, "display_hour should be 0-23, aborting.\n");

	print_usage();
	exit(1);
	}
if((display_minute < 0) || (display_minute > 59) )
	{
	fprintf(stderr, "display_minute should be 0-59, aborting\n");
	
	print_usage();
	exit(1);
	}

if((display_second < 0) || (display_second > 59) )
	{
	fprintf(stderr, "display_econd should be 0-59, aborting.\n");
	
	print_usage();
	exit(1);
	}

/* end sanity checks */


start_seconds = (3600 * hour ) + (60 * minute) + second;
seconds_to_go = start_seconds;


/* clear lead_text array */
for(i = 0; i < MAX_LINES; i++)
	{
	temp1[i][0] = 0;
	}

/* replace any '\n' in the user text by */
lf_flag = 0;
a = 0;
for(i = 0; i < MAX_LINES; i++)
	{
	for(j = 0; j < TEXT_ARRAY_SIZE; j++)
		{
		c = lead_text[a];
		a++;
		
		if(c == '\\')
			{
			lf_flag = 1;
			}

		if(lf_flag)
			{
			if(c == 'n')
				{
				lf_flag = 0;
				temp1[i][j - 1] = 13; // LF
				temp1[i][j    ] = 10; // CR
				temp1[i][j + 1] = 0;
				c = 0;

				break;
				}
			} 

		temp1[i][j] = c;

		if(c == 0)
			{
			break;
			}
		} /* end for all characters in a lead_text line */

	} /* end for all lead_text lines */


if(debug_flag)
	{
	for(i = 0; i < MAX_LINES; i++)
		{
		if(temp1[i][0] != 0)
			{
			fprintf(stderr, "%d %s\n", i, temp1[i]);
			j = 0;
			for(j = 0; j < 40; j++)
				{
				fprintf(stderr, "%02x ", temp1[i][j]);			
				}

			}
		}
	}


/* clear trail_text array */
for(i = 0; i < MAX_LINES; i++)
	{
	temp2[i][0] = 0;
	}

/* replace any '\n' in the user text by */
lf_flag = 0;
a = 0;
for(i = 0; i < MAX_LINES; i++)
	{
	for(j = 0; j < TEXT_ARRAY_SIZE; j++)
		{
		c = trail_text[a];
		a++;
		
		if(c == '\\')
			{
			lf_flag = 1;
			}

		if(lf_flag)
			{
			if(c == 'n')
				{
				lf_flag = 0;
				temp2[i][j - 1] = 13; // LF
				temp2[i][j    ] = 10; // CR
				temp2[i][j + 1] = 0;
				c = 0;

				break;
				}
			} 

		temp2[i][j] = c;

		if(c == 0)
			{
			break;
			}
		} /* end for all characters in a trail_text line */

	} /* end for all lead_text lines */



if(debug_flag)
	{
	for(i = 0; i < MAX_LINES; i++)
		{
		if(temp2[i][0] != 0)
			{
			fprintf(stderr, "%d %s\n", i, temp2[i]);
			j = 0;
			for(j = 0; j < 40; j++)
				{
				fprintf(stderr, "%02x ", temp2[i][j]);			
				}

			}
		}
	}


/* clear end_text array */
for(i = 0; i < MAX_LINES; i++)
	{
	temp3[i][0] = 0;
	}

/* replace any '\n' in the user text by */
lf_flag = 0;
a = 0;
for(i = 0; i < MAX_LINES; i++)
	{
	for(j = 0; j < TEXT_ARRAY_SIZE; j++)
		{
		c = end_text[a];
		a++;
		
		if(c == '\\')
			{
			lf_flag = 1;
			}

		if(lf_flag)
			{
			if(c == 'n')
				{
				lf_flag = 0;
				temp3[i][j - 1] = 13; // LF
				temp3[i][j    ] = 10; // CR
				temp3[i][j + 1] = 0;
				c = 0;

				break;
				}
			} 

		temp3[i][j] = c;

		if(c == 0)
			{
			break;
			}
		} /* end for all characters in a lead_text line */

	} /* end for all lead_text lines */


if(debug_flag)
	{
	for(i = 0; i < MAX_LINES; i++)
		{
		if(temp3[i][0] != 0)
			{
			fprintf(stderr, "%d %s\n", i, temp3[i]);
			j = 0;
			for(j = 0; j < 40; j++)
				{
				fprintf(stderr, "%02x ", temp3[i][j]);			
				}

			}
		}
	}


/* info for user */
fprintf(stderr, "Start time=%02d:%02d:%02d\n", hour, minute, second);

/* srt files seem to start with entry  1 */
entry_number = 1;

/* loop that counts down seconds and writes an .srt entry for each second */
while(1)
	{
/*
SRT example:

4
00:01:29,019 --> 00:01:31,294
Dieu Tout Puissant,
nous sommes enfin libres !
*/

	/* sequence number */
	fprintf(stdout, "%d\r\n", entry_number);

	if(seconds_to_go == 0)
		{
		/* keep end_text on screen for at least an hour */
		
		fprintf(stdout,\
		"%02d:%02d:%02d,%03d --> %02d:%02d:%02d,%03d\r\n",\
		display_hour, display_minute, display_second, display_start_milli_second,\
		display_hour + 2, display_minute, display_second, display_end_milli_second);


		/* print the timer */
		fprintf(stdout, "%02d:%02d:%02d", hour, minute, second);

		if(end_text[0] == 0) fprintf(stdout, "\r\n");

		/* print any end_text line(s) */
		for(i = 0; i < MAX_LINES; i++)
			{
			if(temp3[i][0] == 0) break;
			fprintf(stdout, "%s", temp3[i]);
			}		
		}
	else
		{
		/* print entry */
		fprintf(stdout,\
		"%02d:%02d:%02d,%03d --> %02d:%02d:%02d,%03d\r\n",\
		display_hour, display_minute, display_second, display_start_milli_second,\
		display_hour, display_minute, display_second, display_end_milli_second);

		/* print any lead_text line(s) */
		for(i = 0; i < MAX_LINES; i++)
			{
			if(temp1[i][0] == 0) break;
			fprintf(stdout, "%s", temp1[i]);
			}		

		/* print the timer */
		fprintf(stdout, "%02d:%02d:%02d", hour, minute, second);

		if(trail_text[0] == 0) fprintf(stdout, "\r\n");

		/* print any trail_text line(s) */
		for(i = 0; i < MAX_LINES; i++)
			{
			if(temp2[i][0] == 0) break;
			fprintf(stdout, "%s", temp2[i]);
			}		
		}

	/* empty line ends text, everything in MSDOS format */
	fprintf(stdout, "\r\n\r\n");

	/* test for counter zero */
	seconds_to_go--;
	if(seconds_to_go < 0)
		{
		exit(0);
		}

	/* update things */
	second--;
	if(second <0)
		{
		second = 59;
		minute--;
		if(minute < 0)
			{
			minute = 59;
			hour--;
			if(hour < 0)
				{
				hour = 23;
				}
			}
		}	
	
	display_second++;
	if(display_second == 60)
		{
		display_second = 0;
		display_minute++;
		if(display_minute == 60)
			{
			display_minute = 0;
			display_hour++;
			if(display_hour == 24)
				{
				display_hour = 0;
				}
			}
		}	

	entry_number++;
	} /* end wile count down seconds */

} /* end function main */


